#!%PERL%
# Copyright (c) vhffs project and its contributors
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# 1. Redistributions of source code must retain the above copyright
#   notice, this list of conditions and the following disclaimer.
#2. Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in
#   the documentation and/or other materials provided with the
#   distribution.
#3. Neither the name of vhffs nor the names of its contributors
#   may be used to endorse or promote products derived from this
#   software without specific prior written permission.
#
#THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
#"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
#LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
#FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
#COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
#INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
#BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
#LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
#CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

use strict;
use utf8;
use File::Path;
use File::Basename;

use Vhffs::Constants;
use Vhffs::Functions;
use Vhffs::Robots;
use Vhffs::Services::Mail;

package Vhffs::Robots::Mail;

sub getall_boxes {
	my ( $vhffs, $state ) = @_;

	my @params;

	my $sql = 'SELECT m.domain, m.boxes_path, b.local_part, b.mbox_name, b.state FROM vhffs_boxes b INNER JOIN vhffs_mxdomain m ON m.domain=b.domain';
	if( defined $state)   {
		$sql .= ' WHERE b.state=?';
		push(@params, $state);
	}

	my $request = $vhffs->get_db->prepare( $sql );
	return undef unless $request->execute( @params );
	return $request->fetchall_arrayref({});
}

sub create {
	my $mail = shift;
	return undef unless defined $mail and $mail->get_status == Vhffs::Constants::WAITING_FOR_CREATION;

	my $vhffs = $mail->get_vhffs;

	$mail->set_status( Vhffs::Constants::ACTIVATED );
	$mail->commit;
	Vhffs::Robots::vhffs_log( $vhffs, 'Created mail domain '.$mail->get_domain );

	return 1;
}

sub delete {
	my $mail = shift;
	return undef unless defined $mail and $mail->get_status == Vhffs::Constants::WAITING_FOR_DELETION;

	my $vhffs = $mail->get_vhffs;
	my $dir = $mail->get_dir();

	require Vhffs::Services::MailingList;
	my $lists = Vhffs::Services::MailingList::getall( $vhffs, undef, undef, undef, $mail->get_domain );
	if( @$lists ) {
		foreach( @{$lists} ) {
			next if $_->get_status == Vhffs::Constants::WAITING_FOR_DELETION;
			$_->set_status( Vhffs::Constants::WAITING_FOR_DELETION );
			$_->commit;
		}
		Vhffs::Robots::vhffs_log( $vhffs, 'Cannot delete mail domain '.$mail->get_domain.', remaining mailing lists, requests have been sent to delete them' );
		# Do not call next right now, check for boxes to avoir further failure
	}

	if( $mail->nb_boxes ) {
		foreach ( values( %{$mail->get_boxes} ) ) {
			$mail->set_box_status( $_->{'local_part'}, Vhffs::Constants::WAITING_FOR_DELETION );
		}
		Vhffs::Robots::vhffs_log( $vhffs, 'Cannot delete mail domain '.$mail->get_domain.', '.$mail->nb_boxes.' boxes remaining, requests have been sent to delete them' );
		return 1;
	}

	return 1 if @$lists;

	File::Path::remove_tree( $dir, { error => \my $errors });
	# Mail domain directories are hashed on two levels, so we've potentially two empty directories to delete
	my $parent = File::Basename::dirname($dir);
	rmdir $parent;
	$parent = File::Basename::dirname($parent);
	rmdir $parent;

	if(@$errors) {
		$mail->set_status( Vhffs::Constants::DELETION_ERROR );
		$mail->commit;
		Vhffs::Robots::vhffs_log( $vhffs, 'An error occured while removing mail domain '.$mail->get_domain.' from the filesystem: '.join(', ', @$errors) );
		return undef;
	}

	if( $mail->delete ) {
		Vhffs::Robots::vhffs_log( $vhffs, 'Deleted mail domain '.$mail->get_domain );
	} else {
		Vhffs::Robots::vhffs_log( $vhffs, 'An error occured while deleting mail domain '.$mail->get_domain );
		$mail->set_status( Vhffs::Constants::DELETION_ERROR );
		$mail->commit;
		return undef;
	}

	return 1;
}

sub modify {
	my $mail = shift;
	return undef unless defined $mail and $mail->get_status == Vhffs::Constants::WAITING_FOR_MODIFICATION;
	$mail->set_status( Vhffs::Constants::ACTIVATED );
	$mail->commit;
	return 1;
}

sub create_mailbox {
	my $mail = shift;
	my $localpart = shift;
	return undef unless defined $mail and defined $localpart and $mail->get_box_status($localpart) == Vhffs::Constants::WAITING_FOR_CREATION;

	my $vhffs = $mail->get_vhffs;
	my $mailconf = $vhffs->get_config->get_service('mail');

	my $dir = $mail->get_box_dir($localpart);

	my $prevumask = umask 0;
	File::Path::make_path( $dir, { mode => 0755, error => \my $errors } );
	File::Path::make_path( $dir.'/Maildir/cur', $dir.'/Maildir/new', $dir.'/Maildir/tmp', { mode => 0700, error => \$errors }) unless @$errors;
	umask $prevumask;

	if( @$errors ) {
		$mail->set_box_status( $localpart, Vhffs::Constants::CREATION_ERROR );
		Vhffs::Robots::vhffs_log( $vhffs, 'An error occured while creating mail box '.$localpart.'@'.$mail->get_domain.' to the filesystem: '.join(', ', @$errors) );
		return undef;
	}

	# make_path owner and group directives do not seem to work if owner and group and numeric
	Vhffs::Robots::chown_recur( $dir, $mailconf->{'boxes_uid'}, $mailconf->{'boxes_gid'} );
	chmod 0700, $dir;

	Vhffs::Robots::vhffs_log( $vhffs, 'Created mail box '.$localpart.'@'.$mail->get_domain );
	$mail->set_box_status( $localpart, Vhffs::Constants::ACTIVATED );
	return 1;
}

sub delete_mailbox {
	my $mail = shift;
	my $localpart = shift;
	return undef unless defined $mail and defined $localpart and $mail->get_box_status($localpart) == Vhffs::Constants::WAITING_FOR_DELETION;

	my $vhffs = $mail->get_vhffs;
	my $dir = $mail->get_box_dir($localpart);

	Vhffs::Robots::archive_targz( $mail, $dir, [ $localpart ] );

	File::Path::remove_tree( $dir, { error => \my $errors } );
	# Remove the letter/ directory if empty
	rmdir File::Basename::dirname($dir);

	if( @$errors ) {
		$mail->set_box_status( $localpart, Vhffs::Constants::DELETION_ERROR );
		Vhffs::Robots::vhffs_log( $vhffs, 'An error occured while deleting mail box '.$localpart.'@'.$mail->get_domain.' from the filesystem: '.join(', ', @$errors) );
		return undef;
	}

	Vhffs::Robots::vhffs_log( $vhffs, 'Deleted mail box '.$localpart.'@'.$mail->get_domain );
	return $mail->delbox($localpart);
}

1;
